@@ -6,6 +6,8 @@ PICA::Parser - Parse PICA+ data
 
 =head1 SYNOPSIS
 
+  use PICA::Parser;
+
   PICA::Parser->parsefile( $filename_or_handle ,
       Field => \&field_handler,
       Record => \&record_handler
@@ -16,6 +18,23 @@ PICA::Parser - Parse PICA+ data
       Record => \&record_handler
   );
 
+  $parser = PICA::Parser->new(
+      Record => \&record_handler,
+      Proceed => 1
+  );
+  $parser->parsefile( $filename );
+  $parser->parsedata( $picadata );
+  print $parser->counter() . " records read.\n";
+
+You can also export C<parsedata> and C<parsefile>:
+
+  use PICA::Parser qw(parsefile);
+
+  parsefile( $filename, Record => sub {
+      my $record = shift;
+      print $record->to_string() . "\n";
+  });
+
 =head1 DESCRIPTION
 
 This module can be used to parse normalized PICA+ and PICA+ XML.
@@ -29,12 +48,15 @@ use warnings;
 
 use Carp;
 
-use vars qw($VERSION);
-$VERSION = "0.31";
+use vars qw($VERSION @ISA @EXPORT_OK);
+$VERSION = "0.35";
+
+@ISA = qw(Exporter);
+@EXPORT_OK = qw(parsefile parsedata);
 
 =head1 CONSTRUCTOR
 
-=head2 new (params)
+=head2 new ( [ %params ] )
 
 Creates a Parser to store common parameters (see below). These 
 parameters will be used as default when calling C<parsefile> or
@@ -46,34 +68,6 @@ use C<PICA::Parser>. These two methods do the same:
 
   PICA::Parser->parsefile( $file, %params );
 
-=cut
-
-sub new {
-    my ($class, %params) = @_;
-    $class = ref $class || $class;
-
-    my $self = bless {
-        defaultparams => {},
-        parser => undef
-    }, $class;
-
-    %{ $self->{defaultparams} } = %params if %params;
-
-    return $self;
-}
-
-=head1 METHODS
-
-=head2 parsefile (filename, params)
-
-Parses pica data from a file, specified by a filename or filehandle.
-The default parser is L<PICA::PlainParser>. If the filename extension 
-is C<.xml> or C<.xml.gz> or the 'Format' parameter set to 'xml' then
-L<PICA::XMLParser> is used instead. 
-
-  PICA::Parser->parsefile( "data.picaplus", Field => \&field_handler );
-  PICA::Parser->parsefile( \*STDIN, Field => \&field_handler, Format='XML' );
-
 Common parameters that are passed to the specific parser are:
 
 =over
@@ -110,23 +104,21 @@ Reference to a handler function for parsed PICA+ collections.
 An array of L<PICA::Record> objects is passed to the function.
 For performace reasons it is recommended to directly use the
 stream of records with the L<Record> handler instead of collecting
-all storing records and using them afterwards.
+all storing records and using them afterwards. This parameter is 
+not supported yet.
 
 =item Error
 
 This handler is used if an error occured while parsing, for instance
 if data does not look like PICA+. By default errors are just ignored.
 
-All errors are counted, the number of errors is given in the 
-L<errors> method.
+TODO: Count errors and return the number of errors in the C<errors> method.
 
 =item Strict
 
-TODO: strict_mode only available in PlainParser which needs to be implemented
-TODO: @ISA!
-
 Stop on errors. By default a parser just omits records that could
-not been parsed. (default is false).
+not been parsed. (default is false). Up to now strict_mode is only
+available in L<PICA::PlainParser>!
 
 =item EmptyRecords
 
@@ -138,33 +130,75 @@ case empty records are counted with a special counter that can be read
 with the C<empty> method. The normal counter (method C<counter>) 
 counts all records no matter if empty or not.
 
+=item Proceed
+
+By default the internal counters are reset with each call of C<parsefile> 
+and C<parsedata>. If you set the C<Proceed> parameter to a true value, 
+the same parser will be reused without reseting. 
+
 =back
 
-Specific parser may support additional handlers or parameters.
+=cut
+
+sub new {
+    my $class = "PICA::Parser";
+    if (scalar(@_) % 2) { # odd
+        $class = shift;
+        $class = ref $class || $class;
+    }
+    my %params = @_;
+
+    my $self = bless {
+        defaultparams => {},
+        xmlparser => undef,
+        plainparser => undef
+    }, $class;
+
+    %{ $self->{defaultparams} } = %params if %params;
+
+    return $self;
+}
+
+=head1 METHODS
+
+=head2 parsefile ( $filename-or-handle [, %params ] )
+
+Parses pica data from a file, specified by a filename or filehandle.
+The default parser is L<PICA::PlainParser>. If the filename extension 
+is C<.xml> or C<.xml.gz> or the 'Format' parameter set to 'xml' then
+L<PICA::XMLParser> is used instead. 
+
+  PICA::Parser->parsefile( "data.picaplus", Field => \&field_handler );
+  PICA::Parser->parsefile( \*STDIN, Field => \&field_handler, Format='XML' );
+
+See the constructor C<new> for a description of parameters.
+The C<Proceed> parameter is ignored.
+
+You cannot parse a file named C<"PICA::Parser"> by the way.
 
 =cut
 
 sub parsefile {
-    my ($self, $arg, %params) = @_;
-    %params =  %{ $self->{defaultparams} } if ref $self and not %params;
+    my $self = shift;
+    my ($arg, $parser);
 
-    my $parser;
-
-    if ( ($params{Format} and $params{Format} =~ /^xml$/i) or 
-         (ref(\$arg) eq 'SCALAR' and ($arg =~ /.xml$/i or $arg =~ /.xml.gz$/i)) 
-       ) {
-        require PICA::XMLParser;
-        $parser = PICA::XMLParser->new( %params );
-    } else {
-        require PICA::PlainParser;
-        $parser = PICA::PlainParser->new( %params );
+    if (ref($self) eq 'PICA::Parser') { # called as a method
+        $arg = shift;
+        my %params = @_;
+        if (ref(\$arg) eq 'SCALAR' and ($arg =~ /.xml$/i or $arg =~ /.xml.gz$/i)) {
+            $params{Format} = "XML";
+        }
+        $parser = $self->_getparser( %params );
+    } else { # called as a function
+        $arg = ($self eq 'PICA::Parser') ? shift : $self;
+        $parser = PICA::Parser->new( @_ );
     }
-    $self->{parser} = $parser if ref $self;
 
+    croak("Missing argument to parsefile") unless defined $arg;
     $parser->parsefile( $arg );
 }
 
-=head2 parsedata (data, params)
+=head2 parsedata ( $data [, %params ] )
 
 Parses data from a string, array reference, or function. See
 C<parsefile> and the C<parsedata> method of L<PICA::PlainParser>
@@ -176,21 +210,24 @@ By default L<PICA::PlainParser> is used unless there the
   PICA::Parser->parsedata( $picastring, Field => \&field_handler );
   PICA::Parser->parsedata( \@picalines, Field => \&field_handler );
 
+If data is a L<PICA::Record> object, it is directly passed to the 
+record handler without re-parsing. See the constructor C<new> for 
+a description of parameters. The C<Proceed> parameter is ignored.
+
 =cut
 
 sub parsedata {
-    my ($self, $data, %params) = @_;
-    %params = %{ $self->{defaultparams} } if ref $self and not %params;
+    my $self = shift;
+    my ( $data, $parser );
 
-    my $parser;
-    if ($params{Format} and $params{Format} =~ /^xml$/i) {
-        require PICA::XMLParser;
-        $parser = PICA::XMLParser->new( %params );
-    } else {
-        require PICA::PlainParser;
-        $parser = PICA::PlainParser->new( %params );
+    if (ref($self) eq 'PICA::Parser') { # called as a method
+        $data = shift;
+        my %params = @_;
+        $parser = $self->_getparser( %params );
+    } else { # called as a function
+        $data = ($self eq 'PICA::Parser') ? shift : $self;
+        $parser = PICA::Parser->new( @_ );
     }
-    $self->{parser} = $parser if ref $self;
 
     $parser->parsedata( $data );
 }
@@ -202,11 +239,13 @@ Get the number of read records so far.
 =cut
 
 sub counter {
-   my $self = shift;
-   return undef if !ref $self;
+    my $self = shift;
+    return undef if !ref $self;
 
-   my $parser = $self->{parser};
-   return $parser->counter() if $parser;
+    my $counter = 0;
+    $counter += $self->{plainparser}->counter() if $self->{plainparser};
+    $counter += $self->{xmlparser}->counter() if $self->{xmlparser};
+    return $counter;
 }
 
 =head2 empty
@@ -223,22 +262,60 @@ sub empty {
    my $self = shift; 
    return undef if !ref $self;
 
-   my $parser = $self->{parser};
-   return $parser->empty() if $parser;
+    my $empty = 0;
+    $empty += $self->{plainparser}->empty() if $self->{plainparser};
+    $empty += $self->{xmlparser}->empty() if $self->{xmlparser};
+    return $empty;
 }
 
-=head2 errors
+=head1 INTERNAL METHODS
 
-Return the number of errors that ocurred so far.
+=head2 _getparser ( [ %params] )
+
+Internal method to get a new parser of the internal parser of this object.
+By default, gives a L<PICA:PlainParser> unless you specify the C<Format>
+parameter. Single parameters override the default parameters specified at
+the constructor (except the the C<Proceed> parameter).
 
 =cut
 
-sub errors {
-   my $self = shift;
-   return undef if !ref $self;
+sub _getparser {
+    my $self = shift;
+    my %params = @_;
+    delete $params{Proceed} if defined $params{Proceed};
 
-   my $parser = $self->{parser};
-   return $parser->errors() if $parser;
+    my $parser;
+
+    # join parameters
+    my %unionparams = ();
+    my %defaultparams = %{ $self->{defaultparams} };
+    my $key;
+    foreach $key (keys %defaultparams) {
+        $unionparams{$key} = $defaultparams{$key}
+    }
+    foreach $key (keys %params) {
+        $unionparams{$key} = $params{$key}
+    }
+    # remove format parameter
+    delete $params{Format} if defined $params{Format};
+
+    # XMLParser
+    if ( defined $unionparams{Format} and $unionparams{Format} =~ /^xml$/i ) {
+        if ( !$self->{xmlparser} or %params ) {
+            require PICA::XMLParser; 
+            #if ($self->{xmlparser} && 
+            $self->{xmlparser} = PICA::XMLParser->new( %unionparams );
+        }
+        $parser = $self->{xmlparser};
+    } else { # PlainParser
+        if ( !$self->{plainparser} or %params ) {
+            require PICA::PlainParser; 
+            $self->{plainparser} = PICA::PlainParser->new( %unionparams );
+        }
+        $parser = $self->{plainparser};
+    }
+
+    return $parser;
 }
 
 1;
